@@ -25,9 +25,18 @@ from lib389.rootdse import RootDSE
 from ....conftest import get_rpm_version
 from lib389._mapped_object import DSLdapObjects
 from lib389.replica import Replicas, Changelog
-from lib389.backend import Backends
+from lib389.backend import Backends, BackendSuffixView
 from lib389.idm.domain import Domain
 from lib389.nss_ssl import NssSsl
+from lib389._constants import *
+from lib389 import DirSrv
+from lib389.instance.setup import SetupDs
+from lib389.instance.options import General2Base, Slapd2Base
+import os
+import random
+import ldap
+import time
+import subprocess
 
 
 pytestmark = pytest.mark.tier0
@@ -35,6 +44,7 @@ pytestmark = pytest.mark.tier0
 default_paths = Paths()
 
 log = logging.getLogger(__name__)
+DEBUGGING = os.getenv("DEBUGGING", default=False)
 
 # Globals
 USER1_DN = 'uid=user1,' + DEFAULT_SUFFIX
@@ -50,6 +60,190 @@ ROOTDSE_DEF_ATTR_LIST = ('namingContexts',
                          'vendorName',
                          'vendorVersion')
 
+# This MAX_FDS value left about 22 connections available with bdb
+# (should have more connections with lmdb)
+MAX_FDS = 150
+
+
+
+default_paths = Paths()
+
+
+
+log = logging.getLogger(__name__)
+DEBUGGING = os.getenv("DEBUGGING", default=False)
+
+
+class CustomSetup():
+    DEFAULT_GENERAL = { 'config_version': 2,
+                        'full_machine_name': 'localhost.localdomain',
+                        'strict_host_checking': False,
+                        # Not setting 'systemd' because it is not used.
+                        # (that is the global default.inf setting that matters)
+                      }
+    DEFAULT_SLAPD = { 'root_password': PW_DM,
+                      'defaults': INSTALL_LATEST_CONFIG,
+                    }
+    DEFAULT_BACKENDS = [ { 
+                            'cn': 'userroot',
+                            'nsslapd-suffix': DEFAULT_SUFFIX,
+                            'sample_entries': 'yes',
+                            BACKEND_SAMPLE_ENTRIES: INSTALL_LATEST_CONFIG,
+                       }, ]
+
+    WRAPPER_FORMAT = '''#!/bin/sh
+{wrapper_options}
+exec {nsslapd} -D {cfgdir} -i {pidfile} 
+'''
+
+
+    class CustomDirSrv(DirSrv):
+        def __init__(self, verbose=False, external_log=log):
+            super().__init__(verbose=verbose, external_log=external_log)
+            self.wrapper = None       # placeholder for the wrapper file name
+            
+        def _reset_systemd(self):
+            self.systemd_override = False
+    
+        def status(self):
+            self._reset_systemd()
+            return super().status()
+    
+        def start(self, timeout=120, *args):
+            if self.status():
+                return
+            tmp_env = os.environ
+            # Unset PYTHONPATH to avoid mixing old CLI tools and new lib389
+            if "PYTHONPATH" in tmp_env:
+                del tmp_env["PYTHONPATH"]
+            try:
+                subprocess.check_call([
+                    '/usr/bin/sh',
+                    self.wrapper
+                ], env=tmp_env, stderr=subprocess.STDOUT)
+            except subprocess.CalledProcessError as e:
+                log.fatal("%s failed!  Error (%s) %s" % (self.wrapper, e.returncode, e.output))
+                raise e from None
+            for count in range(timeout):
+                if self.status():
+                    return
+                time.sleep(1)
+            raise TimeoutException('Failed to start ns-slpad')
+    
+        def stop(self, timeout=120):
+            self._reset_systemd()
+            super().stop(timeout=timeout)
+    
+
+    def _search_be(belist, beinfo):
+        for be in belist:
+            if be['cn'] == beinfo['cn']:
+                return be
+        return None
+
+    def __init__(self, serverid, general=None, slapd=None, backends=None, log=log):
+        verbose = log.level > logging.DEBUG
+        self.log = log
+        self.serverid = serverid
+        self.verbose = verbose
+        self.wrapper = f'/tmp/ds_{serverid}_wrapper.sh'
+        if serverid.startswith('slapd-'):
+            self.instname = server.id
+        else:
+            self.instname = 'slapd-'+serverid
+        self.ldapi = None
+        self.pid_file = None
+        self.inst = None
+
+        # Lets prepare the options
+        general_options = General2Base(log)
+        for d in (CustomSetup.DEFAULT_GENERAL, general):
+            if d:
+                for key,val in d.items():
+                    general_options.set(key,val)
+        log.debug('[general]: %s' % general_options._options)
+        self.general = general_options
+        
+        slapd_options = Slapd2Base(self.log)
+        slapd_options.set('instance_name', serverid)
+        for d in (CustomSetup.DEFAULT_SLAPD, slapd):
+            if d:
+                for key,val in d.items():
+                    slapd_options.set(key,val)
+        log.debug('[slapd]: %s' % slapd_options._options)
+        self.slapd = slapd_options
+
+        backend_options = []
+        for backend_list in (CustomSetup.DEFAULT_BACKENDS, backends):
+            if not backend_list:
+                continue
+            for backend in backend_list:
+                target_be = CustomSetup._search_be(backend_options, backend)
+                if not target_be:
+                    target_be = {}
+                    backend_options.append(target_be)
+                for key,val in backend.items():
+                    target_be[key] = val
+        log.debug('[backends]: %s' % backend_options)
+        self.backends = backend_options
+
+    def _to_dirsrv_args(self):
+        args = {}
+        slapd = self.slapd.collect()
+        general = self.general.collect()
+        args["SER_HOST"] = general['full_machine_name']
+        args["SER_PORT"] = slapd['port']
+        args["SER_SECURE_PORT"] = slapd['secure_port']
+        args["SER_SERVERID_PROP"] = self.serverid
+        return args
+	
+    def create_instance(self):
+        sds = SetupDs(verbose=self.verbose, dryrun=False, log=self.log)
+        self.general.verify()
+        general = self.general.collect()
+        self.slapd.verify()
+        slapd = self.slapd.collect()
+        sds.create_from_args(general, slapd, self.backends, None)
+        self.ldapi = get_ldapurl_from_serverid(self.serverid)[0]
+        args = self._to_dirsrv_args()
+        log.debug('DirSrv.allocate args = %s' % str(args))
+        log.debug('ldapi = %s' % str(self.ldapi))
+        root_dn = slapd['root_dn']
+        root_password = slapd['root_password']
+        inst = DirSrv(verbose=self.verbose, external_log=self.log)
+        inst.local_simple_allocate(self.serverid, ldapuri=self.ldapi, binddn=root_dn, password=root_password)
+        self.pid_file = inst.pid_file()
+        # inst.setup_ldapi()
+        log.debug('DirSrv = %s' % str(inst.__dict__))
+        inst.open()
+        inst.stop()
+        inst = CustomSetup.CustomDirSrv(verbose=self.verbose, external_log=self.log)
+        inst.local_simple_allocate(self.serverid, ldapuri=self.ldapi, binddn=root_dn, password=root_password)
+        self.inst = inst
+        return inst
+
+    def create_wrapper(self, maxfds=None):
+        self.inst.wrapper = self.wrapper
+        slapd = self.slapd.collect()
+        sbin_dir = slapd['sbin_dir']
+        config_dir = slapd['config_dir']
+        fmtvalues = {
+            'nsslapd': f'{sbin_dir}/ns-slapd',
+            'cfgdir': config_dir.format(instance_name=self.instname),
+            'pidfile': self.pid_file,
+            'wrapper_options': ''
+        }
+        if maxfds:
+            fmtvalues['wrapper_options']=f'ulimit -n {maxfds}\nulimit -H -n {maxfds}'
+        with open(self.wrapper, 'w') as f:
+            f.write(CustomSetup.WRAPPER_FORMAT.format(**fmtvalues))
+
+    def cleanup(self):
+        self.inst.stop()
+        self.inst.delete()
+        if os.path.exists(self.wrapper):
+            os.remove(self.wrapper)
+
 
 @pytest.fixture(scope="function")
 def _reset_attr(request, topology_st):
@@ -150,6 +344,29 @@ def change_conf_attr(topology_st, suffix, attr_name, attr_value):
     return attr_value_bck
 
 
+@pytest.fixture(scope="function")
+def ldapagent_config(topology_st, request):
+    """Creates agent.conf for snmp agent
+    """
+
+    var_dir = topology_st.standalone.get_local_state_dir()
+    config_file = os.path.join(topology_st.standalone.get_sysconf_dir(), 'dirsrv/config/agent.conf')
+    config = f"""agentx-supplier {var_dir}/agentx/supplier
+agent-logdir {var_dir}/log/dirsrv
+server slapd-{topology_st.standalone.serverid}
+"""
+
+    with open(config_file, 'w') as agent_config_file:
+        agent_config_file.write(config)
+
+    def fin():
+        os.remove(config_file)
+
+    request.addfinalizer(fin)
+
+    return config_file
+
+
 def test_basic_ops(topology_st, import_example_ldif):
     """Tests adds, mods, modrdns, and deletes operations
 
@@ -888,66 +1105,30 @@ def test_basic_referrals(topology_st, import_example_ldif):
     """
 
     log.info('Running test_basic_referrals...')
-    SUFFIX_CONFIG = 'cn="dc=example,dc=com",cn=mapping tree,cn=config'
-    #
-    # Set the referral, and the backend state
-    #
-    try:
-        topology_st.standalone.modify_s(SUFFIX_CONFIG,
-                                        [(ldap.MOD_REPLACE,
-                                          'nsslapd-referral',
-                                          b'ldap://localhost.localdomain:389/o%3dnetscaperoot')])
-    except ldap.LDAPError as e:
-        log.fatal('test_basic_referrals: Failed to set referral: error ' + e.args[0]['desc'])
-        assert False
+    backends = Backends(topology_st.standalone)
+    backend = backends.list()[0]
+    bev = BackendSuffixView(topology_st.standalone, backend)
+    bev.set('nsslapd-referral', 'ldap://localhost.localdomain:389/o%3dnetscaperoot')
+    bev.set('nsslapd-state', 'referral')
 
-    try:
-        topology_st.standalone.modify_s(SUFFIX_CONFIG, [(ldap.MOD_REPLACE,
-                                                         'nsslapd-state', b'Referral')])
-    except ldap.LDAPError as e:
-        log.fatal('test_basic_referrals: Failed to set backend state: error '
-                  + e.args[0]['desc'])
-        assert False
+    log.info('Checking that the settings were applied...')
+    assert bev.get_attr_val_utf8('nsslapd-referral') == 'ldap://localhost.localdomain:389/o%3dnetscaperoot'
+    assert bev.get_attr_val_utf8('nsslapd-state') == 'referral'
 
-    #
-    # Test that a referral error is returned
-    #
+    log.info('Testing that a referral error is returned...')
     topology_st.standalone.set_option(ldap.OPT_REFERRALS, 0)  # Do not follow referral
-    try:
+    with pytest.raises(ldap.REFERRAL):
         topology_st.standalone.search_s(DEFAULT_SUFFIX, ldap.SCOPE_SUBTREE, 'objectclass=top')
-    except ldap.REFERRAL:
-        pass
-    except ldap.LDAPError as e:
-        log.fatal('test_basic_referrals: Search failed: ' + e.args[0]['desc'])
-        assert False
 
-    #
     # Make sure server can restart in referral mode
-    #
+    log.info('Restarting the server...')
     topology_st.standalone.restart(timeout=10)
 
-    #
-    # Cleanup
-    #
-    try:
-        topology_st.standalone.modify_s(SUFFIX_CONFIG, [(ldap.MOD_REPLACE,
-                                                         'nsslapd-state', b'Backend')])
-    except ldap.LDAPError as e:
-        log.fatal('test_basic_referrals: Failed to set backend state: error '
-                  + e.args[0]['desc'])
-        assert False
-
-    try:
-        topology_st.standalone.modify_s(SUFFIX_CONFIG, [(ldap.MOD_DELETE,
-                                                         'nsslapd-referral', None)])
-    except ldap.LDAPError as e:
-        log.fatal('test_basic_referrals: Failed to delete referral: error '
-                  + e.args[0]['desc'])
-        assert False
+    log.info('Cleaning up...')
+    bev.set('nsslapd-state', 'backend')
+    bev.remove_all('nsslapd-referral')
     topology_st.standalone.set_option(ldap.OPT_REFERRALS, 1)
 
-    log.info('test_basic_referrals: PASSED')
-
 
 def test_basic_systemctl(topology_st, import_example_ldif):
     """Tests systemctl/lib389 can stop and start the server.
@@ -1028,7 +1209,7 @@ def test_basic_systemctl(topology_st, import_example_ldif):
     log.info('test_basic_systemctl: PASSED')
 
 
-def test_basic_ldapagent(topology_st, import_example_ldif):
+def test_basic_ldapagent(topology_st, import_example_ldif, ldapagent_config):
     """Tests that the ldap agent starts
 
     :id: da1d1846-8fc4-4b8c-8e53-4c9c16eff1ba
@@ -1048,18 +1229,9 @@ def test_basic_ldapagent(topology_st, import_example_ldif):
 
     if not os.path.exists(os.path.join(topology_st.standalone.get_sbin_dir(), 'ldap-agent')):
         pytest.skip("ldap-agent is not present")
-    var_dir = topology_st.standalone.get_local_state_dir()
-
-    config_file = os.path.join(topology_st.standalone.get_sysconf_dir(), 'dirsrv/config/agent.conf')
-
-    agent_config_file = open(config_file, 'w')
-    agent_config_file.write('agentx-supplier ' + var_dir + '/agentx/supplier\n')
-    agent_config_file.write('agent-logdir ' + var_dir + '/log/dirsrv\n')
-    agent_config_file.write('server slapd-' + topology_st.standalone.serverid + '\n')
-    agent_config_file.close()
 
     # Remember, this is *forking*
-    check_output([os.path.join(topology_st.standalone.get_sbin_dir(), 'ldap-agent'), config_file])
+    check_output([os.path.join(topology_st.standalone.get_sbin_dir(), 'ldap-agent'), ldapagent_config])
     # First kill any previous agents ....
     run_dir = topology_st.standalone.get_run_dir()
     pidpath = os.path.join(run_dir, 'ldap-agent.pid')
@@ -1209,8 +1381,6 @@ def test_basic_anonymous_search(topology_st, create_users):
         assert len(entries) != 0
 
 
-@pytest.mark.ds604
-@pytest.mark.bz915801
 def test_search_original_type(topology_st, create_users):
     """Test ldapsearch returning original attributes
        using nsslapd-search-return-original-type-switch
@@ -1246,7 +1416,6 @@ def test_search_original_type(topology_st, create_users):
     assert "objectclass overflow" not in entries[0].getAttrs()
 
 
-@pytest.mark.bz192901
 def test_search_ou(topology_st):
     """Test that DS should not return an entry that does not match the filter
 
@@ -1342,8 +1511,6 @@ def test_bind_entry_missing_passwd(topology_st):
         user.bind("some_password")
 
 
-@pytest.mark.bz1044135
-@pytest.mark.ds47319
 def test_connection_buffer_size(topology_st):
     """Test connection buffer size adjustable with different values(valid values and invalid)
 
@@ -1367,7 +1534,6 @@ def test_connection_buffer_size(topology_st):
             topology_st.standalone.config.replace('nsslapd-connection-buffer', value)
 
 
-@pytest.mark.bz1637439
 def test_critical_msg_on_empty_range_idl(topology_st):
     """Doing a range index lookup should not report a critical message even if IDL is empty
 
@@ -1440,8 +1606,6 @@ def test_critical_msg_on_empty_range_idl(topology_st):
     assert not topology_st.standalone.searchErrorsLog('CRIT - list_candidates - NULL idl was recieved from filter_candidates_ext.')
 
 
-@pytest.mark.bz1870624
-@pytest.mark.ds4379
 @pytest.mark.parametrize("case,value", [('positive', ['cn','','']),
                                         ("positive", ['cn', '', '', '', '', '', '', '', '', '', '']),
                                         ("negative", ['cn', '', '', '', '', '', '', '', '', '', '', ''])])
@@ -1468,8 +1632,6 @@ def test_attr_description_limit(topology_st, case, value):
             DSLdapObjects(topology_st.standalone, basedn='').filter("(objectclass=*)", attrlist=value, scope=0)
 
 
-@pytest.mark.bz1647099
-@pytest.mark.ds50026
 def test_ldbm_modification_audit_log(topology_st):
     """When updating LDBM config attributes, those attributes/values are not listed
     in the audit log
@@ -2083,8 +2245,6 @@ sample_entries = yes
 
 @pytest.mark.skipif(not get_user_is_root() or ds_is_older('1.4.2.0'),
                     reason="This test is only required with new admin cli, and requires root.")
-@pytest.mark.bz1748016
-@pytest.mark.ds50581
 def test_dscreate_ldapi(dscreate_long_instance):
     """Test that an instance with a long name can
     handle ldapi connection using a long socket name
@@ -2105,8 +2265,6 @@ def test_dscreate_ldapi(dscreate_long_instance):
 
 @pytest.mark.skipif(not get_user_is_root() or ds_is_older('1.4.2.0'),
                     reason="This test is only required with new admin cli, and requires root.")
-@pytest.mark.bz1715406
-@pytest.mark.ds50923
 def test_dscreate_multiple_dashes_name(dscreate_long_instance):
     """Test that an instance with a multiple dashes in the name
     can be removed with dsctl --remove-all
@@ -2175,8 +2333,6 @@ suffix = {request.param}
 
 @pytest.mark.skipif(not get_user_is_root() or ds_is_older('1.4.0.0'),
                     reason="This test is only required with new admin cli, and requires root.")
-@pytest.mark.bz1807419
-@pytest.mark.ds50928
 def test_dscreate_with_different_rdn(dscreate_test_rdn_value):
     """Test that dscreate works with different RDN attributes as suffix
 
@@ -2207,6 +2363,100 @@ def test_dscreate_with_different_rdn(dscreate_test_rdn_value):
             assert True
 
 
+@pytest.fixture(scope="module")
+def dscreate_custom_instance(request):
+    topo = CustomSetup('custom')
+
+    def fin():
+        topo.cleanup()
+
+    request.addfinalizer(fin)
+    topo.create_instance()
+    # Return CustomSetup object associated with 
+    #  a stopped instance named "custom"
+    return topo
+
+    obj.create_wrapper(maxfds=150)
+    log.info("Starting wrapper")
+    inst.start()
+    log.info("Server is started.")
+    log.info("Open connection")
+    inst.open()
+
+
+@pytest.fixture(scope="module", params=set(range(1,5)))
+def dscreate_with_numlistener(request, dscreate_custom_instance):
+    numlisteners = request.param
+    dscreate_custom_instance.create_wrapper(maxfds=MAX_FDS)
+    inst = dscreate_custom_instance.inst
+    inst.stop()
+    dse_ldif = DSEldif(inst)
+    dse_ldif.replace('cn=config', 'nsslapd-numlisteners', str(numlisteners))
+    inst.start()
+    inst.open()
+    return inst
+
+
+@pytest.mark.skipif(ds_is_older('2.2.0.0'),
+                    reason="This test is only required with multiple listener support.")
+def test_conn_limits(dscreate_with_numlistener):
+    """Check the connections limits for various number of listeners.
+
+    :id: 7be2eb5c-4d8f-11ee-ae3d-482ae39447e5
+    :parametrized: yes
+    :setup: Setup an instance then set nsslapd-numlisteners and maximum file descriptors
+    :steps:
+        1. Loops on:
+             Open new connection and perform search until timeout expires
+        2. Close one of the previously open connections
+        3. Loops MAX_FDS times on:
+              - opening a new connection
+              - perform a search
+              - close the connection
+        4. Close all open connections
+        5. Remove the instance
+    :expectedresults:
+        1. Should get a timeout (because the server has no more any connections)
+        2. Should success
+        3. Should success (otherwise issue #5924 has probably been hit)
+        4. Should success
+        5. Should success
+    """
+    inst = dscreate_with_numlistener
+
+    conns = []
+    timeout_occured = False
+    for i in range(MAX_FDS):
+        try:
+            ldc = ldap.initialize(f'ldap://localhost:{inst.port}')
+            ldc.set_option(ldap.OPT_TIMEOUT, 5)
+            ldc.search_s(DEFAULT_SUFFIX, ldap.SCOPE_SUBTREE, "(uid=demo)")
+            conns.append(ldc)
+        except ldap.TIMEOUT:
+            timeout_occured = True
+            break
+    # Should not be able to open MAX_FDS connections (some file descriptor are
+    #  reserved (for example for the listening socket )
+    assert timeout_occured
+
+    conn = random.choice(conns)
+    conn.unbind()
+    conns.remove(conn)
+
+    # Should loop enough time so trigger issue #5924 if it is not fixed.
+    for i in range(MAX_FDS):
+        ldc = ldap.initialize(f'ldap://localhost:{inst.port}')
+        # Set a timeout long enough so that the test fails if server is unresponsive
+        ldc.set_option(ldap.OPT_TIMEOUT, 60)
+        ldc.search_s(DEFAULT_SUFFIX, ldap.SCOPE_SUBTREE, "(uid=demo)")
+        ldc.unbind()
+
+    # Close all open connections
+    for c in conns:
+        c.unbind()
+
+    # Step 6 is done in teardown phase by dscreate_instance finalizer
+
 if __name__ == '__main__':
     # Run isolated
     # -s for DEBUG mode
