@@ -11,6 +11,7 @@ import logging
 import pytest
 import os
 from lib389 import DirSrv, pid_from_file
+from lib389.dseldif import DSEldif
 from lib389.tasks import *
 from lib389.topologies import topology_m2, topology_st as topo
 from lib389.utils import *
@@ -47,8 +48,6 @@ def big_file():
     return TEMP_BIG_FILE
 
 
-@pytest.mark.bz1897248
-@pytest.mark.ds4315
 @pytest.mark.skipif(ds_is_older('1.4.3.16'), reason="This config setting exists in 1.4.3.16 and higher")
 def test_nagle_default_value(topo):
     """Test that nsslapd-nagle attribute is off by default
@@ -204,8 +203,6 @@ def test_config_deadlock_policy(topology_m2):
     ldbmconfig.replace('nsslapd-db-deadlock-policy', deadlock_policy)
 
 
-@pytest.mark.bz766322
-@pytest.mark.ds26
 def test_defaultnamingcontext(topo):
     """Tests configuration attribute defaultNamingContext in the rootdse
 
@@ -290,7 +287,6 @@ def test_defaultnamingcontext(topo):
     b2.delete()
 
 
-@pytest.mark.bz602456
 def test_allow_add_delete_config_attributes(topo):
     """Tests configuration attributes are allowed to add and delete
 
@@ -332,8 +328,6 @@ def test_allow_add_delete_config_attributes(topo):
     assert not topo.standalone.config.present('invalid-attribute', 'invalid-value')
 
 
-@pytest.mark.bz918705
-@pytest.mark.ds511
 def test_ignore_virtual_attrs(topo):
     """Test nsslapd-ignore-virtual-attrs configuration attribute
 
@@ -442,8 +436,6 @@ def test_ignore_virtual_attrs_after_restart(topo):
     log.info("Check the default value of attribute nsslapd-ignore-virtual-attrs should be OFF")
     assert topo.standalone.config.present('nsslapd-ignore-virtual-attrs', 'off')
 
-@pytest.mark.bz918694
-@pytest.mark.ds408
 def test_ndn_cache_enabled(topo):
     """Test nsslapd-ignore-virtual-attrs configuration attribute
 
@@ -750,6 +742,66 @@ def test_lmdb_config(create_lmdb_instance):
     set_and_check(inst, db_config, 'mdb_max_dbs', 'nsslapd-mdb-max-dbs', 200)
 
 
+def test_numlisteners_limit(topo):
+    """Test higher limit of nsslapd-numlisteners than 4
+    DS allows a higher value of nsslapd-numlisteners than it's limit of 4
+
+    :id: 96869ea9-c7b4-4a4f-85f9-ea1d3f4a63aa
+    :setup: Standalone Instance
+    :steps:
+        1. Check default nsslapd-numlisteners value is 1
+        2. Set nsslapd-numlisteners value to 4
+        3. Check nsslapd-numlisteners value is set to 4
+        4. Check dse.ldif value of nsslapd-numlisteners is set to 4
+        5. systemctl restart dirsrv@localhost
+        6. Check nsslapd-numlisteners value is 4, after server restart
+        7. Set nsslapd-numlisteners value to greater than the max value (4)
+        8. Check if nsslapd-numlisteners value is still 4
+        9. Check dse.ldif value of nsslapd-numlisteners is set to 4
+    :expectedresults:
+        1. nsslapd-numlisteners config value should show 1 by default
+        2. nsslapd-numlisteners value should be successfully set to 4
+        3. nsslapd-numlisteners is indeed set to 4
+        4. nsslapd-numlisteners value in localhost dse.ldif file is set to 4
+        5. restart DS instance is successful
+        6. nsslapd-numlisteners value is still 4 after server restart
+        7. Invalid value is rejected
+        8. nsslapd-numlisteners value is still 4
+        9. nsslapd-numlisteners value in localhost dse.ldif file is set to 4
+    """
+    # Check default value for nsslapd-numlisteners is 1
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '1'
+
+    # Set nsslapd-numlisteners value to 4
+    topo.standalone.config.set('nsslapd-numlisteners', '4')
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check instance dse.ldif value is set to 4
+    inst = topo.standalone
+    dse_ldif = DSEldif(inst)
+    numlisteners = dse_ldif.get(DN_CONFIG, 'nsslapd-numlisteners')
+    assert numlisteners[0] == '4'
+
+    # Restart instance
+    topo.standalone.restart()
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check if nsslapd-numlisteners value is not set more than 4
+    with pytest.raises(ldap.UNWILLING_TO_PERFORM):
+        assert(not topo.standalone.config.replace('nsslapd-numlisteners', '5'))
+
+    # Check nsslapd-numlisteners value is set to 4
+    assert topo.standalone.config.get_attr_val_utf8('nsslapd-numlisteners') == '4'
+
+    # Check the value of nsslapd-numlisteners in dse.ldif is set to 4
+    numlisteners = dse_ldif.get(DN_CONFIG, 'nsslapd-numlisteners')
+    assert numlisteners[0] == '4'
+
+
 if __name__ == '__main__':
     # Run isolated
     # -s for DEBUG mode
