@@ -40,6 +40,11 @@ def run_healthcheck_and_flush_log(topology, instance, searched_code, json, searc
     args.check = ['config', 'refint', 'backends', 'monitor-disk-space', 'logs', 'memberof']
     args.dry_run = False
 
+    # If we are using BDB as a backend, we will get error DSBLE0006 on new versions
+    if ds_is_newer("3.0.0") and instance.get_db_lib() == 'bdb' and \
+       (searched_code is CMD_OUTPUT or searched_code is JSON_OUTPUT):
+        searched_code = 'DSBLE0006'
+
     if json:
         log.info('Use healthcheck with --json option')
         args.json = json
@@ -80,8 +85,6 @@ def setup_ldif(topology_st, request):
     request.addfinalizer(fin)
 
 
-@pytest.mark.ds50873
-@pytest.mark.bz1685160
 @pytest.mark.xfail(ds_is_older("1.4.1"), reason="Not implemented")
 def test_healthcheck_logging_format_should_be_revised(topology_st):
     """Check if HealthCheck returns DSCLE0001 code
@@ -124,8 +127,6 @@ def test_healthcheck_logging_format_should_be_revised(topology_st):
     run_healthcheck_and_flush_log(topology_st, standalone, json=True, searched_code=JSON_OUTPUT)
 
 
-@pytest.mark.ds50873
-@pytest.mark.bz1685160
 @pytest.mark.xfail(ds_is_older("1.4.1"), reason="Not implemented")
 def test_healthcheck_RI_plugin_is_misconfigured(topology_st):
     """Check if HealthCheck returns DSRILE0001 code
@@ -173,8 +174,6 @@ def test_healthcheck_RI_plugin_is_misconfigured(topology_st):
     run_healthcheck_and_flush_log(topology_st, standalone, json=True, searched_code=JSON_OUTPUT)
 
 
-@pytest.mark.ds50873
-@pytest.mark.bz1685160
 @pytest.mark.xfail(ds_is_older("1.4.1"), reason="Not implemented")
 def test_healthcheck_RI_plugin_missing_indexes(topology_st):
     """Check if HealthCheck returns DSRILE0002 code
@@ -274,8 +273,6 @@ def test_healthcheck_MO_plugin_missing_indexes(topology_st):
     standalone.restart()
 
 
-@pytest.mark.ds50873
-@pytest.mark.bz1685160
 @pytest.mark.xfail(ds_is_older("1.4.1"), reason="Not implemented")
 def test_healthcheck_virtual_attr_incorrectly_indexed(topology_st):
     """Check if HealthCheck returns DSVIRTLE0001 code
@@ -334,8 +331,6 @@ def test_healthcheck_virtual_attr_incorrectly_indexed(topology_st):
     run_healthcheck_and_flush_log(topology_st, standalone, RET_CODE, json=True)
 
 
-@pytest.mark.ds50873
-@pytest.mark.bz1685160
 @pytest.mark.xfail(ds_is_older("1.4.1"), reason="Not implemented")
 @pytest.mark.xfail(ds_is_older("1.4.2.4"), reason="May fail because of bug 1796050")
 def test_healthcheck_low_disk_space(topology_st):
@@ -381,8 +376,6 @@ def test_healthcheck_low_disk_space(topology_st):
 
 
 @pytest.mark.flaky(max_runs=2, min_passes=1)
-@pytest.mark.ds50791
-@pytest.mark.bz1843567
 @pytest.mark.xfail(ds_is_older("1.4.3.8"), reason="Not implemented")
 def test_healthcheck_notes_unindexed_search(topology_st, setup_ldif):
     """Check if HealthCheck returns DSLOGNOTES0001 code
@@ -435,8 +428,6 @@ def test_healthcheck_notes_unindexed_search(topology_st, setup_ldif):
     run_healthcheck_and_flush_log(topology_st, standalone, RET_CODE, json=True)
 
 
-@pytest.mark.ds50791
-@pytest.mark.bz1843567
 @pytest.mark.xfail(ds_is_older("1.4.3.8"), reason="Not implemented")
 def test_healthcheck_notes_unknown_attribute(topology_st, setup_ldif):
     """Check if HealthCheck returns DSLOGNOTES0002 code
@@ -519,7 +510,7 @@ def test_healthcheck_unauth_binds(topology_st):
     inst.config.set("nsslapd-allow-unauthenticated-binds", "off")
 
 def test_healthcheck_accesslog_buffering(topology_st):
-    """Check if HealthCheck returns DSCLE0004 code when acccess log biffering
+    """Check if HealthCheck returns DSCLE0004 code when acccess log buffering
     is disabled
 
     :id: 5a6512fd-1c7b-4557-9278-45150423148b
@@ -551,7 +542,7 @@ def test_healthcheck_accesslog_buffering(topology_st):
     inst.config.set("nsslapd-accesslog-logbuffering", "on")
 
 def test_healthcheck_securitylog_buffering(topology_st):
-    """Check if HealthCheck returns DSCLE0005 code when security log biffering
+    """Check if HealthCheck returns DSCLE0005 code when security log buffering
     is disabled
 
     :id: 9b84287a-e022-4bdc-8c65-2276b37371b5
@@ -582,6 +573,41 @@ def test_healthcheck_securitylog_buffering(topology_st):
     log.info('Reset nnsslapd-securitylog-logbuffering to on')
     inst.config.set("nsslapd-securitylog-logbuffering", "on")
 
+def test_healthcheck_auditlog_buffering(topology_st):
+    """Check if HealthCheck returns DSCLE0006 code when audit log buffering
+    is disabled
+
+    :id: f030c9f3-0ce7-4156-ba70-81ef3ac82867
+    :setup: Standalone instance
+    :steps:
+        1. Create DS instance
+        2. Set nsslapd-auditlog-logbuffering to off
+        3. Use HealthCheck without --json option
+        4. Use HealthCheck with --json option
+    :expectedresults:
+        1. Success
+        2. Success
+        3. Healthcheck reports DSCLE0006
+        4. Healthcheck reports DSCLE0006
+    """
+
+    RET_CODE = 'DSCLE0006'
+
+    inst = topology_st.standalone
+    enabled = inst.config.get_attr_val_utf8('nsslapd-auditlog-logging-enabled')
+
+    log.info('nsslapd-auditlog-logbuffering to off')
+    inst.config.set('nsslapd-auditlog-logging-enabled', 'on')
+    inst.config.set('nsslapd-auditlog-logbuffering', 'off')
+
+    run_healthcheck_and_flush_log(topology_st, inst, RET_CODE, json=False)
+    run_healthcheck_and_flush_log(topology_st, inst, RET_CODE, json=True)
+
+    # reset setting
+    log.info('Reset nnsslapd-auditlog-logbuffering to on')
+    inst.config.set('nsslapd-auditlog-logbuffering', 'on')
+    inst.config.set('nsslapd-auditlog-logging-enabled', enabled)
+
 
 if __name__ == '__main__':
     # Run isolated
