@@ -18,8 +18,11 @@
 
 #if defined(HPUX11) || defined(OS_solaris) || defined(linux)
 /* built-in 64-bit file I/O support */
+#if ! defined(__LP64__)
+/* But not on 64-bit arch: It is needless and build fails since gcc 13.2.1-4 */
 #define DB_USE_64LFS
 #endif
+#endif
 
 /* needed by at least HPUX and Solaris, to define off64_t */
 #ifdef DB_USE_64LFS
@@ -831,6 +834,7 @@ typedef struct _back_search_result_set
 
 #define LDBM_ANCESTORID_STR                "ancestorid"
 #define LDBM_ENTRYDN_STR                   SLAPI_ATTR_ENTRYDN
+#define LDBM_LONG_ENTRYRDN_STR             "@long-entryrdn"
 #define LDBM_ENTRYRDN_STR                  "entryrdn"
 #define LDBM_NUMSUBORDINATES_STR           "numsubordinates"
 #define LDBM_TOMBSTONE_NUMSUBORDINATES_STR "tombstonenumsubordinates"
