@@ -154,7 +154,7 @@ dbmdb_import_update_entry_subcount(backend *be, ID parentid, size_t sub_count, i
     /* If it is a tombstone entry, add tombstonesubordinates instead of
      * numsubordinates. */
     if (slapi_entry_flag_is_set(e->ep_entry, SLAPI_ENTRY_FLAG_TOMBSTONE)) {
-        numsub_str = tombstone_numsubordinates;
+        numsub_str = LDBM_TOMBSTONE_NUMSUBORDINATES_STR;
     }
     /* attr numsubordinates/tombstonenumsubordinates could already exist in
      * the entry, let's check whether it's already there or not */
@@ -668,6 +668,43 @@ error:
 
 /* Helper function to make up filenames */
 
+/* Ensure that the task status and exit code is properly set */
+static void
+dbmdb_task_finish(ImportJob *job, int ret)
+{
+    ldbm_instance *inst = job->inst;
+    char *opname = "importing";
+    char *task_dn = "";
+
+    if (job->flags & (FLAG_DRYRUN | FLAG_UPGRADEDNFORMAT_V1)) {
+        opname = "upgrading dn";
+    } else if (job->flags & FLAG_REINDEXING) {
+        opname = "indexing";
+    }
+
+    if (job->task != NULL) {
+        /*
+         * freeipa expect that finished tasks have both a status and
+         * an exit code.
+         * So lets ensure that both are set.
+         */
+        if (!job->task_status) {
+            dbmdb_import_log_status_start(job);
+        }
+        dbmdb_import_log_status_add_line(job, "%s: Finished %s task",
+                                         inst->inst_name, opname);
+        dbmdb_import_log_status_done(job);
+        slapi_task_finish(job->task, ret);
+        task_dn = slapi_ch_smprintf(" task '%s'", job->task->task_dn);
+    }
+    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_task_finish",
+                  "%s: Finished %s%s. Exit code is %d\n",
+                  inst->inst_name, opname, task_dn, ret);
+    if (*task_dn) {
+        slapi_ch_free_string(&task_dn);
+    }
+}
+
 /* when the import is done, this function is called to bring stuff back up.
  * returns 0 on success; anything else is an error
  */
@@ -698,8 +735,26 @@ dbmdb_import_all_done(ImportJob *job, int ret)
             /* Reset USN slapi_counter with the last key of the entryUSN index */
             ldbm_set_last_usn(inst->inst_be);
 
-            /* bring backend online again */
-            slapi_mtn_be_enable(inst->inst_be);
+            /* Bring backend online again:
+             * In lmdb case, the import framework is also used for reindexing
+             * while in bdb case reindexing uses its own code.
+             * So dbmdb_import_all_done is called either after 
+             * dbmdb_ldif2db or after dbmdb_db2index while
+             * bdb_import_all_done is only called after bdb_ldif2db.
+             *
+             * dbmdb_db2index uses instance_set_busy_and_readonly 
+             * while dbmdb_ldif2db uses slapi_mtn_be_disable
+             * and these functions have to be reverted accordingly.
+             */
+            if (job->flags & FLAG_REINDEXING) {
+                instance_set_not_busy(inst);
+            } else {
+                slapi_mtn_be_enable(inst->inst_be);
+            }
+            slapi_log_err(SLAPI_LOG_INFO, "dbmdb_import_all_done",
+                          "Backend %s is now online.\n",
+                          slapi_be_get_name(inst->inst_be));
+
         }
         ret |= rc;
     }
@@ -708,12 +763,30 @@ dbmdb_import_all_done(ImportJob *job, int ret)
      * (to avoid race condition in CI)
      */
     if ((job->task != NULL) && (0 == slapi_task_get_refcount(job->task))) {
-        slapi_task_finish(job->task, ret & ~WARN_SKIPPED_IMPORT_ENTRY);
+        dbmdb_task_finish(job, ret & ~WARN_SKIPPED_IMPORT_ENTRY);
     }
 
     return ret;
 }
 
+/* vlv_getindices callback that truncate vlv index (in reindex case) */
+static int
+truncate_index_dbi(struct attrinfo *ai, ImportCtx_t *ctx)
+{
+    int rc = 0;
+    if (is_reindexed_attr(ai->ai_type, ctx, ctx->indexVlvs)) {
+        backend *be = ctx->job->inst->inst_be;
+        dbmdb_dbi_t *dbi = NULL;
+        rc = dbmdb_open_dbi_from_filename(&dbi, be, ai->ai_type, ai, MDB_TRUNCATE_DBI);
+        if (!rc) {
+            char *dbname = dbmdb_recno_cache_get_dbname(dbi->dbname);
+            rc = dbmdb_open_dbi_from_filename(&dbi, be, dbname, ai, MDB_TRUNCATE_DBI);
+            slapi_ch_free_string(&dbname);
+        }
+    }
+    return rc;
+}
+
 int
 dbmdb_public_dbmdb_import_main(void *arg)
 {
@@ -777,6 +850,8 @@ dbmdb_public_dbmdb_import_main(void *arg)
             pthread_cond_signal(&job->wire_cv);
             pthread_mutex_unlock(&job->wire_lock);
             break;
+        case IM_INDEX:
+            vlv_getindices((IFP)truncate_index_dbi, ctx, job->inst->inst_be);
         default:
             break;
     }
@@ -982,15 +1057,11 @@ error:
             ret |= WARN_UPGRADE_DN_FORMAT_SPACE;
         } else {
             ret = -1;
-            if (job->task != NULL) {
-                slapi_task_finish(job->task, ret);
-            }
+            dbmdb_task_finish(job, ret);
         }
     } else if (0 != ret) {
         import_log_notice(job, SLAPI_LOG_ERR, "dbmdb_public_dbmdb_import_main", "%s failed.", opstr);
-        if (job->task != NULL) {
-            slapi_task_finish(job->task, ret);
-        }
+        dbmdb_task_finish(job, ret);
     } else {
         if (job->task) {
             /* set task warning if there are no errors */
@@ -1041,6 +1112,28 @@ dbmdb_import_main(void *arg)
     g_decr_active_threadcnt();
 }
 
+static char *
+get_vlv_dbname(const char *attrname)
+{
+    /* Returns vlv index database name as stored in attribute names
+     * freed by the caller .
+     */
+    char *dbname = slapi_ch_malloc(4+strlen(attrname)+1);
+    char *p = dbname;
+    *p++ = 'v';
+    *p++ = 'l';
+    *p++ = 'v';
+    *p++ = '#';
+    for (; *attrname; attrname++) {
+        if (isalnum(*attrname)) {
+            *p = TOLOWER(*attrname);
+            p++;
+        }
+    }
+    *p = '\0';
+    return dbname;
+}
+
 void
 process_db2index_attrs(Slapi_PBlock *pb, ImportCtx_t *ctx)
 {
@@ -1067,7 +1160,7 @@ process_db2index_attrs(Slapi_PBlock *pb, ImportCtx_t *ctx)
             slapi_ch_array_add(&ctx->indexAttrs, slapi_ch_strdup(attrs[i] + 1));
             break;
         case 'T': /* VLV Search to index */
-            slapi_ch_array_add(&ctx->indexAttrs, slapi_ch_strdup(attrs[i] + 1));
+            slapi_ch_array_add(&ctx->indexVlvs, get_vlv_dbname(attrs[i] + 1));
             break;
         }
     }
@@ -1178,6 +1271,14 @@ dbmdb_run_ldif2db(Slapi_PBlock *pb)
         slapi_task_set_cancel_fn(job->task, dbmdb_import_task_abort);
         job->flags |= FLAG_ONLINE;
 
+        if (job->flags & FLAG_REINDEXING) {
+            /* Reindexing task : so we are called by task_index_thread
+             *  that runs in a dedicated thread.
+             * ==> should process the "import" synchroneously.
+             */
+            return dbmdb_public_dbmdb_import_main((void *)job);
+        }
+
         /* create thread for dbmdb_import_main, so we can return */
         thread = PR_CreateThread(PR_USER_THREAD, dbmdb_import_main, (void *)job,
                                  PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
@@ -1300,11 +1401,7 @@ dbmdb_bulk_import_start(Slapi_PBlock *pb)
     dbmdb_delete_instance_dir(be);
     /* it's okay to fail -- it might already be gone */
 
-    /* vlv_init should be called before dbmdb_instance_start
-     * so the vlv dbi get created
-     */
-    vlv_init(job->inst);
-    /* dbmdb_instance_start will init the id2entry index. */
+    /* dbmdb_instance_start will init the id2entry index and the vlv search list. */
     /* it also (finally) fills in inst_dir_name */
     ret = dbmdb_instance_start(be, DBLAYER_IMPORT_MODE);
     if (ret != 0)
@@ -1462,6 +1559,7 @@ dbmdb_ldbm_back_wire_import(Slapi_PBlock *pb)
 
     if (state == SLAPI_BI_STATE_ADD) {
         Slapi_Entry *pb_import_entry = NULL;
+        char buf[BUFSIZ] = "";
         slapi_pblock_get(pb, SLAPI_BULK_IMPORT_ENTRY, &pb_import_entry);
         /* continuing previous import */
         if (!dbmdb_import_entry_belongs_here(pb_import_entry, job->inst->inst_be)) {
@@ -1473,9 +1571,17 @@ dbmdb_ldbm_back_wire_import(Slapi_PBlock *pb)
             return 0;
         }
 
+        if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+            /*
+             * Queued entries are processed (then freed) by another thread
+             * so the sdn should be capured before queuing the entry to avoid
+             * race condition.
+             */
+            escape_string(slapi_sdn_get_dn(slapi_entry_get_sdn(pb_import_entry)), buf);
+        }
         rc = dbmdb_bulk_import_queue(job, pb_import_entry);
-        slapi_log_err(SLAPI_LOG_REPL, "dbmdb_ldbm_back_wire_import", "dbmdb_bulk_import_queue returned %d with entry %s\n",
-                      rc, slapi_sdn_get_dn(slapi_entry_get_sdn(pb_import_entry)));
+        slapi_log_err(SLAPI_LOG_REPL, "dbmdb_ldbm_back_wire_import",
+                      "dbmdb_bulk_import_queue returned %d with entry %s\n", rc, buf);
         return rc;
     }
 
