@@ -84,6 +84,12 @@ static int dbmdb_force_checkpoint(struct ldbminfo *li);
 
 static const char *backupfilelists[] = { INFOFILE, DBMAPFILE, DSE_INSTANCE, DSE_INDEX, NULL };
 
+/*
+ * if ATTRINFO_DEBUG_DELAY > 0
+ * log info message are logged periodically when waiting for busy attrinfo.
+ */
+#define ATTRINFO_DEBUG_DELAY 0 /* delay between messages in seconds */
+
 /*
  * return nsslapd-db-home-directory (dbmdb_dbhome_directory), if exists.
  * Otherwise, return nsslapd-directory (dbmdb_home_directory).
@@ -434,6 +440,9 @@ dbmdb_rm_db_file(backend *be, struct attrinfo *a, PRBool use_lock, int no_force_
     struct ldbminfo *li = NULL;
     ldbm_instance *inst = NULL;
     dblayer_handle *handle = NULL;
+#if ATTRINFO_DEBUG_DELAY > 0
+    time_t attrinfo_log_time = 0;
+#endif
     char *dbname = NULL;
     int rc = 0;
     dbi_db_t *db = 0;
@@ -478,6 +487,16 @@ dbmdb_rm_db_file(backend *be, struct attrinfo *a, PRBool use_lock, int no_force_
                  */
                 PR_ASSERT(a->ai_indexmask & INDEX_OFFLINE);
                 PR_Unlock(inst->inst_handle_list_mutex);
+#if ATTRINFO_DEBUG_DELAY > 0
+                if (attrinfo_log_time == 0) {
+                    attrinfo_log_time = slapi_current_utc_time() + ATTRINFO_DEBUG_DELAY;
+                } else if (attrinfo_log_time < slapi_current_utc_time()) {
+                    slapi_log_err(SLAPI_LOG_INFO, "dbmdb_rm_db_file",
+                                  "Waiting that index %s get released, count=%ld.\n",
+                                  a->ai_type, a->ai_dblayer_count);
+                    attrinfo_log_time = slapi_current_utc_time() + ATTRINFO_DEBUG_DELAY;
+                }
+#endif
                 DS_Sleep(DBLAYER_CACHE_DELAY);
                 PR_Lock(inst->inst_handle_list_mutex);
             }
@@ -792,7 +811,7 @@ dbmdb_copyfile(char *source, char *destination, int overwrite __attribute__((unu
     int dest_fd = -1;
     char *buffer = NULL;
     int return_value = -1;
-    int bytes_to_write = 0;
+    size_t bytes_to_write = 0;
 
     /* malloc the buffer */
     buffer = slapi_ch_malloc(64 * 1024);
@@ -837,12 +856,12 @@ dbmdb_copyfile(char *source, char *destination, int overwrite __attribute__((unu
                 break;
             } else {
                 /* means error */
-                slapi_log_err(SLAPI_LOG_ERR, "dbmdb_copyfile", "Failed to write by \"%s\"; real: %d bytes, exp: %d bytes\n",
+                slapi_log_err(SLAPI_LOG_ERR, "dbmdb_copyfile", "Failed to write by \"%s\"; real: %d bytes, exp: %lu bytes\n",
                               strerror(errno), return_value, bytes_to_write);
                 if (return_value > 0) {
                     bytes_to_write -= return_value;
                     ptr += return_value;
-                    slapi_log_err(SLAPI_LOG_NOTICE, "dbmdb_copyfile", "Retrying to write %d bytes\n", bytes_to_write);
+                    slapi_log_err(SLAPI_LOG_NOTICE, "dbmdb_copyfile", "Retrying to write %lu bytes\n", bytes_to_write);
                 } else {
                     break;
                 }
@@ -963,6 +982,9 @@ dbmdb_backup(struct ldbminfo *li, char *dest_dir, Slapi_Task *task)
     if (ldbm_archive_config(dest_dir, task) != 0) {
         slapi_log_err(SLAPI_LOG_ERR, "dbmdb_backup",
                 "Backup of config files failed or is incomplete\n");
+         if (0 == return_value) {
+            return_value = -1;
+        }
     }
 
     goto bail;
@@ -1440,23 +1462,20 @@ dbmdb_back_ctrl(Slapi_Backend *be, int cmd, void *info)
         struct ldbminfo *li = (struct ldbminfo *)be->be_database->plg_private;
 
         ldbm_instance *inst = (ldbm_instance *) be->be_instance_info;
-        if (li) {
-            dblayer_private *priv = (dblayer_private *)li->li_dblayer_private;
-            if (priv && priv->dblayer_env) {
-                char *instancedir;
-                dbmdb_dbi_t *dbi = NULL;
+        if (li && MDB_CONFIG(li)) {
+            char *instancedir;
+            dbmdb_dbi_t *dbi = NULL;
 
-                slapi_back_get_info(be, BACK_INFO_INSTANCE_DIR, (void **)&instancedir);
-                rc = dbmdb_open_dbi_from_filename(&dbi, be, BDB_CL_FILENAME, NULL, 0);
-                if (rc == MDB_NOTFOUND) {
-                    /* Nothing to do */
-                    rc = 0;
-                } else if (rc == 0) {
-                    rc = dbmdb_dbi_remove(MDB_CONFIG(li), (dbi_db_t**)&dbi);
-                }
-                inst->inst_changelog = NULL;
-                slapi_ch_free_string(&instancedir);
+            slapi_back_get_info(be, BACK_INFO_INSTANCE_DIR, (void **)&instancedir);
+            rc = dbmdb_open_dbi_from_filename(&dbi, be, BDB_CL_FILENAME, NULL, 0);
+            if (rc == MDB_NOTFOUND) {
+                /* Nothing to do */
+                rc = 0;
+            } else if (rc == 0) {
+                rc = dbmdb_dbi_remove(MDB_CONFIG(li), (dbi_db_t**)&dbi);
             }
+            inst->inst_changelog = NULL;
+            slapi_ch_free_string(&instancedir);
         }
         break;
     }
@@ -2113,7 +2132,7 @@ void *dbmdb_recno_cache_build(void *arg)
         }
     }
     while (rc == 0) {
-        slapi_log_err(SLAPI_LOG_INFO, "dbmdb_recno_cache_build", "recno=%d\n", recno);
+        slapi_log_err(SLAPI_LOG_DEBUG, "dbmdb_recno_cache_build", "recno=%d\n", recno);
         if (recno % RECNO_CACHE_INTERVAL != 1) {
             recno++;
             rc = MDB_CURSOR_GET(txn_ctx.cursor, &key, &data, MDB_NEXT);
@@ -2300,11 +2319,21 @@ int dbmdb_cursor_get_recno(dbi_cursor_t *cursor, MDB_val *dbmdb_key, MDB_val *db
     MDB_cursor *newcur = NULL;
     int cmpres = 0;
     int rc = 0;
+    struct ldbminfo *li = (struct ldbminfo *)cursor->be->be_database->plg_private;
+    dbmdb_ctx_t *ctx = MDB_CONFIG(li);
+    dbmdb_dbi_t *dbi = &ctx->dbi_slots[mdb_cursor_dbi(cursor->cur)];
 
     rc = MDB_CURSOR_GET(cursor->cur, &curpos_key, &curpos_data, MDB_GET_CURRENT);
     if (rc != 0) {
         return rc;
     }
+    if (dbi->dbname[0] == '~') {
+        /* Already hanfling the vlv cache (probably running dbscan) */
+        *dbmdb_key = curpos_key;
+        *dbmdb_data = curpos_data;
+        return rc;
+    }
+
     dbmdb_generate_recno_cache_key_by_data(&cache_key, &curpos_key, &curpos_data);
 
     rc = dbmdb_recno_cache_lookup(cursor, &cache_key, &rce);
@@ -2312,17 +2341,17 @@ int dbmdb_cursor_get_recno(dbi_cursor_t *cursor, MDB_val *dbmdb_key, MDB_val *db
         rc = MDB_CURSOR_OPEN(mdb_cursor_txn(cursor->cur), mdb_cursor_dbi(cursor->cur), &newcur);
     }
     if (rc == 0) {
-        rc = MDB_CURSOR_GET(newcur, &rce->key, &rce-> data, MDB_SET);
+        rc = MDB_CURSOR_GET(newcur, &rce->key, &rce->data, MDB_SET);
     }
     while (rc == 0) {
         cmpres = dbmdb_cmp_dbi_record(mdb_cursor_dbi(cursor->cur), &curpos_key, &curpos_data, &rce->key, &rce->data);
-        if (cmpres >= 0) {
+        if (cmpres <= 0) {
             break;
         }
         rce->recno++;
         rc = MDB_CURSOR_GET(newcur, &rce->key, &rce->data, MDB_NEXT);
     }
-    if (cmpres > 0) {
+    if (cmpres < 0) {
         rc = MDB_NOTFOUND;
     }
     if (rc == 0) {
@@ -2343,12 +2372,12 @@ int dbmdb_cursor_set_recno(dbi_cursor_t *cursor, MDB_val *dbmdb_key, MDB_val *db
     MDB_val cache_key = {0};
     dbi_recno_t recno;
     int rc;
-    if (!dbmdb_data || !dbmdb_data->mv_data) {
-        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_cursor_set_recno", "invalid dbmdb_data parameter (should be a dbi_recno_t)\n");
+    if (!dbmdb_key || !dbmdb_key->mv_data) {
+        slapi_log_err(SLAPI_LOG_ERR, "dbmdb_cursor_set_recno", "invalid dbmdb_key parameter (should be a dbi_recno_t)\n");
         return DBI_RC_INVALID;
     }
 
-    memcpy(&recno, dbmdb_data->mv_data, sizeof (dbi_recno_t));
+    memcpy(&recno, dbmdb_key->mv_data, sizeof (dbi_recno_t));
     dbmdb_generate_recno_cache_key_by_recno(&cache_key, recno);
     rc = dbmdb_recno_cache_lookup(cursor, &cache_key, &rce);
     if (rc ==0) {
@@ -2836,7 +2865,7 @@ out:
 int
 dbmdb_public_clear_vlv_cache(Slapi_Backend *be, dbi_txn_t *txn, dbi_db_t *db)
 {
-    char *rcdbname = slapi_ch_smprintf("%s%s", RECNOCACHE_PREFIX, ((dbmdb_dbi_t*)db)->dbname);
+    char *rcdbname = dbmdb_recno_cache_get_dbname(((dbmdb_dbi_t*)db)->dbname);
     dbmdb_dbi_t *rcdbi = NULL;
     MDB_val ok = { 0 };
     int rc = 0;
