@@ -336,15 +336,14 @@ be initialized given the new entry. Added write lock. */
 
 
 void
-vlv_grok_new_import_entry(const struct backentry *e, backend *be)
+vlv_grok_new_import_entry(const struct backentry *e, backend *be, int *seen_them_all)
 {
     struct vlvSearch *p = NULL;
-    static int seen_them_all = 0;
     int any_not_done = 0;
 
 
     slapi_rwlock_wrlock(be->vlvSearchList_lock);
-    if (seen_them_all) {
+    if (*seen_them_all) {
         slapi_rwlock_unlock(be->vlvSearchList_lock);
         return;
     }
@@ -362,11 +361,62 @@ vlv_grok_new_import_entry(const struct backentry *e, backend *be)
             }
         }
     if (!any_not_done) {
-        seen_them_all = 1;
+        *seen_them_all = 1;
     }
     slapi_rwlock_unlock(be->vlvSearchList_lock);
 }
 
+void
+vlv_rebuild_scope_filter(backend *be)
+{
+    ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+    struct vlvSearch *p = NULL;
+    back_txn new_txn = {NULL};
+    back_txn *txn = NULL;
+    Slapi_PBlock *pb;
+
+    txn = dblayer_get_pvt_txn(); /* Let reuse existing txn if possible */
+    if (!txn && dblayer_read_txn_begin(be, NULL, &new_txn) == 0) {
+            txn = &new_txn;
+    }
+    pb = slapi_pblock_new();
+    slapi_search_internal_set_pb(pb, "", 0, NULL, NULL, 0, NULL, NULL,
+                                 (void *)plugin_get_default_component_id(), 0);
+    slapi_pblock_set(pb, SLAPI_BACKEND, be);
+    slapi_pblock_set(pb, SLAPI_PLUGIN, be->be_database);
+    if (txn) {
+        slapi_pblock_set(pb, SLAPI_TXN, txn->back_txn_txn);
+    }
+
+    slapi_rwlock_wrlock(be->vlvSearchList_lock);
+    for (p = be->vlvSearchList; p != NULL; p = p->vlv_next) {
+        if (p->vlv_scope != LDAP_SCOPE_ONELEVEL) {
+            /* Only the LDAP_SCOPE_ONELEVEL needs to be rebuild as
+             * they have parentid = baseentryid in their filter
+             */
+            continue;
+        }
+        p->vlv_initialized = 0;
+        if (!slapi_sdn_isempty(p->vlv_base)) {
+            struct backentry *e = NULL;
+            entry_address addr;
+            addr.sdn = p->vlv_base;
+            addr.uniqueid = NULL;
+            e = find_entry(pb, be, &addr, txn, NULL);
+            if (NULL != e) {
+                vlvSearch_reinit(p, e);
+                CACHE_RETURN(&inst->inst_cache, &e);
+                p->vlv_initialized = 1;
+            }
+        }
+    }
+    slapi_rwlock_unlock(be->vlvSearchList_lock);
+    if (txn == &new_txn) {
+        dblayer_txn_abort(be, txn);
+    }
+    slapi_pblock_destroy(pb);
+}
+
 void
 vlv_close(ldbm_instance *inst)
 {
@@ -377,6 +427,55 @@ vlv_close(ldbm_instance *inst)
     }
 }
 
+/*
+ * List vlv filenames without acceding to the vlv target
+ *  backend (unlike vlv_init).
+ * Note that vlv configuration is not fully checked so it is
+ * possible to get names that are not assiciated to working index.
+ * (Useless empty mdb db files are created in such case)
+ */
+char **
+vlv_list_filenames(ldbm_instance *inst)
+{
+    /* The FE DSE *must* be initialised before we get here */
+    const char *indexfilter = "(objectclass=vlvindex)";
+    char * attrs[] = { (char*)type_vlvName, NULL };
+    Slapi_Entry **entries = NULL;
+    Slapi_PBlock *tmp_pb;
+    char *basedn = NULL;
+    char **names = NULL;
+
+    if (!inst) {
+        return names;
+    }
+
+    basedn = slapi_create_dn_string("cn=%s,cn=%s,cn=plugins,cn=config",
+                                    inst->inst_name, inst->inst_li->li_plugin->plg_name);
+    if (NULL == basedn) {
+        return names;
+    }
+
+    tmp_pb = slapi_search_internal(basedn, LDAP_SCOPE_SUBTREE, indexfilter, NULL, attrs, 0);
+    slapi_pblock_get(tmp_pb, SLAPI_PLUGIN_INTOP_SEARCH_ENTRIES, &entries);
+    for (size_t i = 0; entries && entries[i] != NULL; i++) {
+        const char *name = slapi_entry_attr_get_ref(entries[i], type_vlvName);
+        char *filename = name ? vlvIndex_build_filename(name) : NULL;
+        if (filename) {
+            charray_add(&names, filename);
+        }
+    }
+    slapi_free_search_results_internal(tmp_pb);
+    slapi_pblock_destroy(tmp_pb);
+    slapi_ch_free_string(&basedn);
+    return names;
+}
+
+int
+does_vlv_need_init(ldbm_instance *inst)
+{
+    return (inst && inst->inst_be->vlvSearchList_lock == NULL);
+}
+
 /*
  * Search for the VLV entries which describe the pre-computed indexes we
  * support.  Register administartion DSE callback functions.
@@ -788,7 +887,8 @@ do_vlv_update_index(back_txn *txn, struct ldbminfo *li, Slapi_PBlock *pb, struct
     } else {
         /* Very bad idea to do this outside of a transaction */
     }
-    if (priv->dblayer_clear_vlv_cache_fn) {
+    if (txn && !txn->back_special_handling_fn && priv->dblayer_clear_vlv_cache_fn) {
+        /* If there is a txn and it is not an import pseudo txn then clear the vlv cache */
         priv->dblayer_clear_vlv_cache_fn(be, db_txn, db);
     }
     data.size = sizeof(entry->ep_id);
