@@ -21,6 +21,8 @@ import {
 import { log_cmd } from "../../lib/tools.jsx";
 import PropTypes from "prop-types";
 
+const _ = cockpit.gettext;
+
 export class Ciphers extends React.Component {
     constructor(props) {
         super(props);
@@ -38,24 +40,24 @@ export class Ciphers extends React.Component {
         };
 
         // Allow Ciphers
-        this.onAllowCipherToggle = isAllowCipherOpen => {
+        this.handleAllowCipherToggle = isAllowCipherOpen => {
             this.setState({
                 isAllowCipherOpen
             });
         };
-        this.onAllowCipherClear = () => {
+        this.handleAllowCipherClear = () => {
             this.setState({
                 allowCiphers: [],
                 isAllowCipherOpen: false
             });
         };
 
-        this.onDenyCipherToggle = isDenyCipherOpen => {
+        this.handleDenyCipherToggle = isDenyCipherOpen => {
             this.setState({
                 isDenyCipherOpen
             });
         };
-        this.onDenyCipherClear = () => {
+        this.handleDenyCipherClear = () => {
             this.setState({
                 denyCiphers: [],
                 isDenyCipherOpen: false
@@ -75,16 +77,16 @@ export class Ciphers extends React.Component {
         const availableCiphers = this.props.supportedCiphers.slice(0); // Copy array
 
         // Parse SSL cipher attributes (nsSSL3Ciphers)
-        if (this.props.cipherPref != "") {
+        if (this.props.cipherPref !== "") {
             let rawCiphers = this.props.cipherPref.split(",");
 
             // First check the first element as it has special meaning
-            if (rawCiphers[0].toLowerCase() == "default") {
+            if (rawCiphers[0].toLowerCase() === "default") {
                 rawCiphers.shift();
-            } else if (rawCiphers[0].toLowerCase() == "+all") {
+            } else if (rawCiphers[0].toLowerCase() === "+all") {
                 cipherPref = "+all";
                 rawCiphers.shift();
-            } else if (rawCiphers[0].toLowerCase() == "-all") {
+            } else if (rawCiphers[0].toLowerCase() === "-all") {
                 cipherPref = "-all";
                 rawCiphers.shift();
             }
@@ -132,13 +134,13 @@ export class Ciphers extends React.Component {
         }
 
         this.setState({
-            cipherPref: cipherPref,
+            cipherPref,
             _cipherPref: cipherPref,
             allowCiphers: allowedCiphers,
             _allowCiphers: [...allowedCiphers],
             denyCiphers: deniedCiphers,
             _denyCiphers: [...deniedCiphers],
-            availableCiphers: availableCiphers,
+            availableCiphers,
         });
     }
 
@@ -167,11 +169,11 @@ export class Ciphers extends React.Component {
                 .done(() => {
                     this.props.addNotification(
                         "success",
-                        `Successfully set cipher preferences.`
+                        _("Successfully set cipher preferences.")
                     );
                     this.props.addNotification(
                         "warning",
-                        `You must restart the Directory Server for these changes to take effect.`
+                        _("You must restart the Directory Server for these changes to take effect.")
                     );
                     this.setState({
                         saving: false,
@@ -187,7 +189,7 @@ export class Ciphers extends React.Component {
                     }
                     this.props.addNotification(
                         "error",
-                        `Error setting cipher preferences - ${msg}`
+                        cockpit.format(_("Error setting cipher preferences - $0"), msg)
                     );
                     this.setState({
                         saving: false,
@@ -199,17 +201,17 @@ export class Ciphers extends React.Component {
     handlePrefChange (val) {
         let disableSaveBtn = true;
 
-        if (JSON.stringify(this.state.allowCiphers) != JSON.stringify(this.state._allowCiphers) ||
-            JSON.stringify(this.state.denyCiphers) != JSON.stringify(this.state._denyCiphers)) {
+        if (JSON.stringify(this.state.allowCiphers) !== JSON.stringify(this.state._allowCiphers) ||
+            JSON.stringify(this.state.denyCiphers) !== JSON.stringify(this.state._denyCiphers)) {
             disableSaveBtn = false;
         }
-        if (this.state._cipherPref != val) {
+        if (this.state._cipherPref !== val) {
             disableSaveBtn = false;
         }
 
         this.setState({
             cipherPref: val,
-            disableSaveBtn: disableSaveBtn,
+            disableSaveBtn,
         });
     }
 
@@ -217,7 +219,7 @@ export class Ciphers extends React.Component {
         let disableSaveBtn = true;
         const availableCiphers = [...this.state.availableCiphers];
 
-        if (this.state.cipherPref != this.state._cipherPref) {
+        if (this.state.cipherPref !== this.state._cipherPref) {
             disableSaveBtn = false;
         }
 
@@ -233,7 +235,7 @@ export class Ciphers extends React.Component {
             if (index > -1) {
                 copy_new_ciphers.splice(index, 1);
             }
-            if (JSON.stringify(this.state._allowCiphers) != JSON.stringify(copy_new_ciphers)) {
+            if (JSON.stringify(this.state._allowCiphers) !== JSON.stringify(copy_new_ciphers)) {
                 disableSaveBtn = false;
             }
 
@@ -241,8 +243,8 @@ export class Ciphers extends React.Component {
                 (prevState) => ({
                     allowCiphers: prevState.allowCiphers.filter((item) => item !== selection),
                     isAllowCipherOpen: false,
-                    availableCiphers: availableCiphers,
-                    disableSaveBtn: disableSaveBtn,
+                    availableCiphers,
+                    disableSaveBtn,
                 }),
             );
         } else {
@@ -252,15 +254,15 @@ export class Ciphers extends React.Component {
                 availableCiphers.splice(index, 1);
                 availableCiphers.sort();
             }
-            if (JSON.stringify(this.state._allowCiphers) != JSON.stringify([...this.state.allowCiphers, selection])) {
+            if (JSON.stringify(this.state._allowCiphers) !== JSON.stringify([...this.state.allowCiphers, selection])) {
                 disableSaveBtn = false;
             }
             this.setState(
                 (prevState) => ({
                     allowCiphers: [...prevState.allowCiphers, selection],
                     isAllowCipherOpen: false,
-                    availableCiphers: availableCiphers,
-                    disableSaveBtn: disableSaveBtn,
+                    availableCiphers,
+                    disableSaveBtn,
                 }),
             );
         }
@@ -270,7 +272,7 @@ export class Ciphers extends React.Component {
         let disableSaveBtn = true;
         const availableCiphers = [...this.state.availableCiphers];
 
-        if (this.state.cipherPref != this.state._cipherPref) {
+        if (this.state.cipherPref !== this.state._cipherPref) {
             disableSaveBtn = false;
         }
 
@@ -286,7 +288,7 @@ export class Ciphers extends React.Component {
             if (index > -1) {
                 copy_new_ciphers.splice(index, 1);
             }
-            if (JSON.stringify(this.state._denyCiphers) != JSON.stringify(copy_new_ciphers)) {
+            if (JSON.stringify(this.state._denyCiphers) !== JSON.stringify(copy_new_ciphers)) {
                 disableSaveBtn = false;
             }
 
@@ -294,8 +296,8 @@ export class Ciphers extends React.Component {
                 (prevState) => ({
                     denyCiphers: prevState.denyCiphers.filter((item) => item !== selection),
                     isDenyCipherOpen: false,
-                    availableCiphers: availableCiphers,
-                    disableSaveBtn: disableSaveBtn,
+                    availableCiphers,
+                    disableSaveBtn,
                 }),
             );
         } else {
@@ -305,7 +307,7 @@ export class Ciphers extends React.Component {
                 availableCiphers.splice(index, 1);
                 availableCiphers.sort();
             }
-            if (JSON.stringify(this.state._denyCiphers) != JSON.stringify([...this.state.denyCiphers, selection])) {
+            if (JSON.stringify(this.state._denyCiphers) !== JSON.stringify([...this.state.denyCiphers, selection])) {
                 disableSaveBtn = false;
             }
 
@@ -313,8 +315,8 @@ export class Ciphers extends React.Component {
                 (prevState) => ({
                     denyCiphers: [...prevState.denyCiphers, selection],
                     isDenyCipherOpen: false,
-                    availableCiphers: availableCiphers,
-                    disableSaveBtn: disableSaveBtn,
+                    availableCiphers,
+                    disableSaveBtn,
                 }),
             );
         }
@@ -324,11 +326,11 @@ export class Ciphers extends React.Component {
         const supportedCiphers = [];
         const enabledCiphers = [];
         let cipherPage;
-        let saveBtnName = "Save Settings";
+        let saveBtnName = _("Save Settings");
         const extraPrimaryProps = {};
         if (this.state.saving) {
-            saveBtnName = "Saving settings ...";
-            extraPrimaryProps.spinnerAriaValueText = "Saving";
+            saveBtnName = _("Saving settings ...");
+            extraPrimaryProps.spinnerAriaValueText = _("Saving");
         }
 
         for (const cipher of this.props.supportedCiphers) {
@@ -343,49 +345,50 @@ export class Ciphers extends React.Component {
         let supportedList = supportedCiphers.map((name) =>
             <SimpleListItem key={name}>{name}</SimpleListItem>
         );
-        if (supportedList.length == 0) {
+        if (supportedList.length === 0) {
             supportedList = "";
         }
         let enabledList = enabledCiphers.map((name) =>
             <SimpleListItem key={name}>{name}</SimpleListItem>
         );
-        if (enabledList.length == 0) {
+        if (enabledList.length === 0) {
             enabledList = "";
         }
 
         if (this.state.saving) {
-            cipherPage =
+            cipherPage = (
                 <div className="ds-center ds-margin-top-lg">
                     <TextContent>
                         <Text component={TextVariants.h3}>
-                            Saving Cipher Preferences ...
+                            {_("Saving Cipher Preferences ...")}
                         </Text>
                     </TextContent>
                     <Spinner size="lg" />
-                </div>;
+                </div>
+            );
         } else {
-            cipherPage =
+            cipherPage = (
                 <div className="ds-indent">
                     <Form className="ds-margin-top-lg" isHorizontal>
                         <Grid>
-                            <GridItem span={5} title="The current ciphers the server is accepting.  This list is only updated after a server restart.">
+                            <GridItem span={5} title={_("The current ciphers the server is accepting.  This list is only updated after a server restart.")}>
                                 <TextContent>
                                     <Text component={TextVariants.h3}>
-                                        Enabled Ciphers <font size="2">({enabledList.length})</font>
+                                        {_("Enabled Ciphers")} <font size="2">({enabledList.length})</font>
                                     </Text>
                                 </TextContent>
                                 <hr />
                                 <div className="ds-box">
-                                    <SimpleList aria-label="enabled cipher list">
+                                    <SimpleList aria-label={_("enabled cipher list")}>
                                         {enabledList}
                                     </SimpleList>
                                 </div>
                             </GridItem>
                             <GridItem span={2} />
-                            <GridItem span={5} title="The current ciphers the server supports">
+                            <GridItem span={5} title={_("The current ciphers the server supports")}>
                                 <TextContent>
                                     <Text component={TextVariants.h3}>
-                                        Supported Ciphers <font size="2">({supportedList.length})</font>
+                                        {_("Supported Ciphers")} <font size="2">({supportedList.length})</font>
                                     </Text>
                                 </TextContent>
                                 <hr />
@@ -399,7 +402,7 @@ export class Ciphers extends React.Component {
                         <hr />
                         <Grid>
                             <GridItem className="ds-label" span={2}>
-                                Cipher Suite
+                                {_("Cipher Suite")}
                             </GridItem>
                             <GridItem span={10}>
                                 <FormSelect
@@ -408,32 +411,32 @@ export class Ciphers extends React.Component {
                                     onChange={this.handlePrefChange}
                                     aria-label="pref select"
                                 >
-                                    <FormSelectOption key="1" value="default" label="Default Ciphers" />
-                                    <FormSelectOption key="2" value="+all" label="All Ciphers" />
-                                    <FormSelectOption key="3" value="-all" label="No Ciphers" />
+                                    <FormSelectOption key="1" value="default" label={_("Default Ciphers")} />
+                                    <FormSelectOption key="2" value="+all" label={_("All Ciphers")} />
+                                    <FormSelectOption key="3" value="-all" label={_("No Ciphers")} />
                                 </FormSelect>
                                 <FormHelperText isHidden={this.state.cipherPref !== "default"}>
-                                    Default cipher suite is chosen. It enables the default ciphers advertised by NSS except weak ciphers.{(this.state.allowCiphers.length !== 0 || this.state.denyCiphers.length !== 0) ? " Any data in the 'Allow Specific Ciphers' and 'Deny Specific Ciphers' fields will be cleaned after the restart." : ""}
+                                    {_("Default cipher suite is chosen. It enables the default ciphers advertised by NSS except weak ciphers.")}{(this.state.allowCiphers.length !== 0 || this.state.denyCiphers.length !== 0) ? _(" Any data in the 'Allow Specific Ciphers' and 'Deny Specific Ciphers' fields will be cleaned after the restart.") : ""}
                                 </FormHelperText>
                             </GridItem>
                         </Grid>
                         <Grid>
                             <GridItem className="ds-label" span={2}>
-                                Allow Specific Ciphers
+                                {_("Allow Specific Ciphers")}
                             </GridItem>
                             <GridItem span={10}>
                                 <Select
                                     variant={SelectVariant.typeaheadMulti}
-                                    typeAheadAriaLabel="Type a cipher"
+                                    typeAheadAriaLabel={_("Type a cipher")}
                                     isDisabled={this.state.cipherPref === "default"}
-                                    onToggle={this.onAllowCipherToggle}
+                                    onToggle={this.handleAllowCipherToggle}
                                     onSelect={this.handleAllowCipherChange}
-                                    onClear={this.onAllowCipherClear}
+                                    onClear={this.handleAllowCipherClear}
                                     selections={this.state.allowCiphers}
                                     isOpen={this.state.isAllowCipherOpen}
                                     aria-labelledby="typeAhead-allow-cipher"
-                                    placeholderText="Type a cipher..."
-                                    noResultsFoundText="There are no matching entries"
+                                    placeholderText={_("Type a cipher...")}
+                                    noResultsFoundText={_("There are no matching entries")}
                                     maxHeight="200px"
                                 >
                                     {this.state.availableCiphers.map((cipher, index) => (
@@ -447,21 +450,21 @@ export class Ciphers extends React.Component {
                         </Grid>
                         <Grid>
                             <GridItem className="ds-label" span={2}>
-                                Deny Specific Ciphers
+                                {_("Deny Specific Ciphers")}
                             </GridItem>
                             <GridItem span={10}>
                                 <Select
                                     variant={SelectVariant.typeaheadMulti}
-                                    typeAheadAriaLabel="Type a cipher"
+                                    typeAheadAriaLabel={_("Type a cipher")}
                                     isDisabled={this.state.cipherPref === "default"}
-                                    onToggle={this.onDenyCipherToggle}
+                                    onToggle={this.handleDenyCipherToggle}
                                     onSelect={this.handleDenyCipherChange}
-                                    onClear={this.onDenyCipherClear}
+                                    onClear={this.handleDenyCipherClear}
                                     selections={this.state.denyCiphers}
                                     isOpen={this.state.isDenyCipherOpen}
                                     aria-labelledby="typeAhead-allow-deny"
-                                    placeholderText="Type a cipher..."
-                                    noResultsFoundText="There are no matching entries"
+                                    placeholderText={_("Type a cipher...")}
+                                    noResultsFoundText={_("There are no matching entries")}
                                     maxHeight="200px"
                                 >
                                     {this.state.availableCiphers.map((cipher, index) => (
@@ -482,12 +485,13 @@ export class Ciphers extends React.Component {
                         }}
                         isDisabled={this.state.disableSaveBtn || this.state.saving}
                         isLoading={this.state.saving}
-                        spinnerAriaValueText={this.state.saving ? "Saving" : undefined}
+                        spinnerAriaValueText={this.state.saving ? _("Saving") : undefined}
                         {...extraPrimaryProps}
                     >
                         {saveBtnName}
                     </Button>
-                </div>;
+                </div>
+            );
         }
 
         return (
