@@ -35,6 +35,9 @@ import PropTypes from "prop-types";
 const settings_attrs = [
     'nsslapd-auditlog',
     'nsslapd-auditlog-logging-enabled',
+    'nsslapd-auditlog-logbuffering',
+    'nsslapd-auditlog-log-format',
+    'nsslapd-auditlog-time-format',
 ];
 
 const rotation_attrs = [
@@ -79,7 +82,7 @@ export class ServerAuditLog extends React.Component {
             attrs: this.props.attrs,
             displayAttrs: [],
             isDisplayAttrOpen: false,
-            attributes: [],
+            attributes: this.props.displayAttrs,
             displayAllAttrs: false,
         };
 
@@ -152,36 +155,12 @@ export class ServerAuditLog extends React.Component {
     componentDidMount() {
         // Loading config
         if (!this.state.loaded) {
-            this.getAttributes();
+            this.loadConfig();
         } else {
             this.props.enableTree();
         }
     }
 
-    getAttributes() {
-        const attr_cmd = [
-            "dsconf",
-            "-j",
-            "ldapi://%2fvar%2frun%2fslapd-" + this.props.serverId + ".socket",
-            "schema",
-            "attributetypes",
-            "list"
-        ];
-        log_cmd("getAttributes", "Get attributes for audit log display attributes", attr_cmd);
-        cockpit
-                .spawn(attr_cmd, { superuser: true, err: "message" })
-                .done(content => {
-                    const attrContent = JSON.parse(content);
-                    const attrs = [];
-                    for (const content of attrContent.items) {
-                        attrs.push(content.name[0]);
-                    }
-                    this.setState({
-                        attributes: attrs,
-                    }, () => { this.loadConfig() });
-                });
-    }
-
     validateSaveBtn(nav_tab, attr, value) {
         let disableSaveBtn = true;
         let disableBtnName = "";
@@ -365,6 +344,7 @@ export class ServerAuditLog extends React.Component {
                     const attrs = config.attrs;
                     let enabled = false;
                     let compressed = false;
+                    let buffering = false;
                     let display_attrs = [];
                     let displayAllAttrs = this.state.displayAllAttrs;
 
@@ -374,6 +354,9 @@ export class ServerAuditLog extends React.Component {
                     if (attrs['nsslapd-auditlog-compress'][0] === "on") {
                         compressed = true;
                     }
+                    if (attrs['nsslapd-auditlog-logbuffering'][0] === "on") {
+                        buffering = true;
+                    }
                     if ('nsslapd-auditlog-display-attrs' in attrs) {
                         if (attrs['nsslapd-auditlog-display-attrs'][0] === "*") {
                             displayAllAttrs = true;
@@ -403,6 +386,9 @@ export class ServerAuditLog extends React.Component {
                         'nsslapd-auditlog-maxlogsize': attrs['nsslapd-auditlog-maxlogsize'][0],
                         'nsslapd-auditlog-maxlogsperdir': attrs['nsslapd-auditlog-maxlogsperdir'][0],
                         'nsslapd-auditlog-compress': compressed,
+                        'nsslapd-auditlog-logbuffering': buffering,
+                        'nsslapd-auditlog-log-format': attrs['nsslapd-auditlog-log-format'][0],
+                        'nsslapd-auditlog-time-format': attrs['nsslapd-auditlog-time-format'][0],
                         displayAttrs: display_attrs,
                         displayAllAttrs,
                         // Record original values
@@ -420,6 +406,9 @@ export class ServerAuditLog extends React.Component {
                         '_nsslapd-auditlog-maxlogsize': attrs['nsslapd-auditlog-maxlogsize'][0],
                         '_nsslapd-auditlog-maxlogsperdir': attrs['nsslapd-auditlog-maxlogsperdir'][0],
                         '_nsslapd-auditlog-compress': compressed,
+                        '_nsslapd-auditlog-logbuffering': buffering,
+                        '_nsslapd-auditlog-log-format': attrs['nsslapd-auditlog-log-format'][0],
+                        '_nsslapd-auditlog-time-format': attrs['nsslapd-auditlog-time-format'][0],
                         _displayAttrs: display_attrs,
                         _displayAllAttrs: displayAllAttrs,
                     });
@@ -441,6 +430,7 @@ export class ServerAuditLog extends React.Component {
         const attrs = this.state.attrs;
         let enabled = false;
         let compressed = false;
+        let buffering = false;
         let display_attrs = [];
         let displayAllAttrs = this.state.displayAllAttrs;
 
@@ -450,6 +440,9 @@ export class ServerAuditLog extends React.Component {
         if (attrs['nsslapd-auditlog-compress'][0] === "on") {
             compressed = true;
         }
+        if (attrs['nsslapd-auditlog-logbuffering'][0] === "on") {
+            buffering = true;
+        }
 
         if ('nsslapd-auditlog-display-attrs' in attrs) {
             if (attrs['nsslapd-auditlog-display-attrs'][0] === "*") {
@@ -479,6 +472,9 @@ export class ServerAuditLog extends React.Component {
             'nsslapd-auditlog-maxlogsize': attrs['nsslapd-auditlog-maxlogsize'][0],
             'nsslapd-auditlog-maxlogsperdir': attrs['nsslapd-auditlog-maxlogsperdir'][0],
             'nsslapd-auditlog-compress': compressed,
+            'nsslapd-auditlog-logbuffering': buffering,
+            'nsslapd-auditlog-log-format': attrs['nsslapd-auditlog-log-format'][0],
+            'nsslapd-auditlog-time-format': attrs['nsslapd-auditlog-time-format'][0],
             displayAttrs: display_attrs,
             displayAllAttrs,
             // Record original values,
@@ -496,6 +492,9 @@ export class ServerAuditLog extends React.Component {
             '_nsslapd-auditlog-maxlogsize': attrs['nsslapd-auditlog-maxlogsize'][0],
             '_nsslapd-auditlog-maxlogsperdir': attrs['nsslapd-auditlog-maxlogsperdir'][0],
             '_nsslapd-auditlog-compress': compressed,
+            '_nsslapd-auditlog-logbuffering': buffering,
+            '_nsslapd-auditlog-log-format': attrs['nsslapd-auditlog-log-format'][0],
+            '_nsslapd-auditlog-time-format': attrs['nsslapd-auditlog-time-format'][0],
             _displayAttrs: display_attrs,
             _displayAllAttrs: displayAllAttrs,
         }, this.props.enableTree);
@@ -540,7 +539,7 @@ export class ServerAuditLog extends React.Component {
                             title={_("Enable audit logging (nsslapd-auditlog-logging-enabled).")}
                             label={_("Enable Audit Logging")}
                         />
-                        <Form className="ds-margin-top-xlg ds-margin-left" isHorizontal autoComplete="off">
+                        <Form className="ds-margin-top-lg ds-left-margin-md" isHorizontal autoComplete="off">
                             <FormGroup
                                 label={_("Audit Log Location")}
                                 fieldId="nsslapd-auditlog"
@@ -557,8 +556,42 @@ export class ServerAuditLog extends React.Component {
                                     }}
                                 />
                             </FormGroup>
+                            <FormGroup
+                                label="Time Format"
+                                fieldId="nsslapd-auditlog-time-format"
+                                title="Time format using strftime formatting (nsslapd-auditlog-time-format)."
+                            >
+                                <TextInput
+                                    value={this.state['nsslapd-auditlog-time-format']}
+                                    type="text"
+                                    id="nsslapd-auditlog-time-format"
+                                    aria-describedby="horizontal-form-name-helper"
+                                    name="nsslapd-auditlog-time-format"
+                                    onChange={(str, e) => {
+                                        this.handleChange(e, "settings");
+                                    }}
+                                />
+                            </FormGroup>
+                            <FormGroup
+                                label="Log Format"
+                                fieldId="nsslapd-auditlog-log-format"
+                                title="Choose the log format (nsslapd-auditlog-log-format)."
+                            >
+                                <FormSelect
+                                    id="nsslapd-auditlog-log-format"
+                                    value={this.state['nsslapd-auditlog-log-format']}
+                                    onChange={(str, e) => {
+                                        this.handleChange(e, "settings");
+                                    }}
+                                    aria-label="FormSelect Input"
+                                >
+                                    <FormSelectOption key="0" value="default" label="Default" />
+                                    <FormSelectOption key="1" value="json" label="JSON" />
+                                    <FormSelectOption key="2" value="json-pretty" label="JSON (pretty)" />
+                                </FormSelect>
+                            </FormGroup>
                         </Form>
-                        <Form className="ds-margin-top-lg ds-margin-left" isHorizontal autoComplete="off">
+                        <Form className="ds-margin-top-lg ds-left-margin-md" isHorizontal autoComplete="off">
                             <FormGroup
                                 label={_("Display Attributes")}
                                 fieldId="nsslapd-auditlog-display-attrs"
@@ -597,6 +630,16 @@ export class ServerAuditLog extends React.Component {
                                 />
                             </FormGroup>
                         </Form>
+                        <Checkbox
+                            className="ds-left-margin-md ds-margin-top-lg"
+                            id="nsslapd-auditlog-logbuffering"
+                            isChecked={this.state['nsslapd-auditlog-logbuffering']}
+                            onChange={(checked, e) => {
+                                this.handleChange(e, "settings");
+                            }}
+                            title={_("This applies to both the audit & auditfail logs.  Disable audit log buffering for faster troubleshooting, but this will impact server performance (nsslapd-auditlog-logbuffering).")}
+                            label={_("Audit Log Buffering Enabled")}
+                        />
                         <Button
                             key="save settings"
                             isDisabled={this.state.saveSettingsDisabled || this.state.loading}
@@ -886,9 +929,11 @@ ServerAuditLog.propTypes = {
     addNotification: PropTypes.func,
     serverId: PropTypes.string,
     attrs: PropTypes.object,
+    displayAttrs: PropTypes.array,
 };
 
 ServerAuditLog.defaultProps = {
     serverId: "",
     attrs: {},
+    displayAttrs: [],
 };
