@@ -7,7 +7,7 @@
 # --- END COPYRIGHT BLOCK ---
 
 from lib389.idm.organizationalunit import OrganizationalUnit, OrganizationalUnits, MUST_ATTRIBUTES
-from lib389.cli_base import populate_attr_arguments, _generic_modify
+from lib389.cli_base import populate_attr_arguments, _generic_modify, CustomHelpFormatter
 from lib389.cli_idm import (
     _generic_list,
     _generic_get,
@@ -56,35 +56,35 @@ def rename(inst, basedn, log, args, warn=True):
     _generic_rename(inst, basedn, log.getChild('_generic_rename'), MANY, rdn, args)
 
 def create_parser(subparsers):
-    ou_parser = subparsers.add_parser('organizationalunit', aliases=['ou'], help='Manage organizational units')
+    ou_parser = subparsers.add_parser('organizationalunit', aliases=['ou'], help='Manage organizational units', formatter_class=CustomHelpFormatter)
 
     subcommands = ou_parser.add_subparsers(help='action')
 
-    list_parser = subcommands.add_parser('list', help='list')
+    list_parser = subcommands.add_parser('list', help='list', formatter_class=CustomHelpFormatter)
     list_parser.set_defaults(func=list)
 
-    get_parser = subcommands.add_parser('get', help='get')
+    get_parser = subcommands.add_parser('get', help='get', formatter_class=CustomHelpFormatter)
     get_parser.set_defaults(func=get)
     get_parser.add_argument('selector', nargs='?', help='The term to search for')
 
-    get_dn_parser = subcommands.add_parser('get_dn', help='get_dn')
+    get_dn_parser = subcommands.add_parser('get_dn', help='get_dn', formatter_class=CustomHelpFormatter)
     get_dn_parser.set_defaults(func=get_dn)
     get_dn_parser.add_argument('dn', nargs='?', help='The dn to get')
 
-    create_parser = subcommands.add_parser('create', help='create')
+    create_parser = subcommands.add_parser('create', help='create', formatter_class=CustomHelpFormatter)
     create_parser.set_defaults(func=create)
     populate_attr_arguments(create_parser, MUST_ATTRIBUTES)
 
-    delete_parser = subcommands.add_parser('delete', help='deletes the object')
+    delete_parser = subcommands.add_parser('delete', help='deletes the object', formatter_class=CustomHelpFormatter)
     delete_parser.set_defaults(func=delete)
     delete_parser.add_argument('dn', nargs='?', help='The dn to delete')
 
-    modify_parser = subcommands.add_parser('modify', help='modify <add|delete|replace>:<attribute>:<value> ...')
+    modify_parser = subcommands.add_parser('modify', help='modify <add|delete|replace>:<attribute>:<value> ...', formatter_class=CustomHelpFormatter)
     modify_parser.set_defaults(func=modify)
     modify_parser.add_argument('selector', nargs=1, help='The %s to modify' % RDN)
     modify_parser.add_argument('changes', nargs='+', help="A list of changes to apply in format: <add|delete|replace>:<attribute>:<value>")
 
-    rename_parser = subcommands.add_parser('rename', help='rename the object')
+    rename_parser = subcommands.add_parser('rename', help='rename the object', formatter_class=CustomHelpFormatter)
     rename_parser.set_defaults(func=rename)
     rename_parser.add_argument('selector', help='The %s to rename' % RDN)
     rename_parser.add_argument('new_name', help='A new organizational unit name')
